import os
import shutil
import subprocess
from multiprocessing import Pool
from pathlib import Path

import typer

from .download import download_all, get_last_modified_date, has_new_crawl

app = typer.Typer()


@app.command()
def download(
    restart: bool = typer.Option(
        False, help="Restart the download of files that failed previously."
    )
):
    """
    Use esse comando para baixar os dados do site da Receita Federal Brasileira.
    """
    typer.echo("Verificando se existe algo novo na Receita")
    if has_new_crawl():
        typer.echo("Novo crawleamento disponível, iniciando...")
        download_all(restart)
    else:
        typer.echo("Nenhum novo dado disponível")
        raise typer.Abort()


@app.command()
def upload_zip():
    """
    Use esse comando para enviar o diretório zippado (como foi baixado) para o Google Cloud Storage
    """
    timestamp = get_last_modified_date()
    typer.echo(f"Enviando diretório para GCS com data: {timestamp}")
    subprocess.check_output(
        f"gsutil -m cp zip/*.zip gs://driva-lake/crawlers/RFB/{timestamp}/zip",
        shell=True,
    )


@app.command()
def extract():
    """
    Use esse comando para extrair os arquivos baixados.
    """
    typer.echo("Extraindo arquivos")
    os.makedirs("extracted", exist_ok=True)
    files = [f"zip/{f}" for f in os.listdir("zip") if f.endswith(".zip")]
    with Pool(processes=4) as pool:
        pool.map(extract_file, files)


def extract_file(file):
    subprocess.check_output(f"unzip {file} -d extracted", shell=True)


@app.command()
def combine():
    """
    Use esse comando para processar o diretório zippado gerando o diretório extraído
    """
    typer.echo(f"Combinando dados")
    script_path = Path(__file__).parent / "scripts/combine.sh"
    subprocess.check_output(
        f"chmod +x {script_path} && {script_path}", shell=True, cwd="extracted"
    )


@app.command()
def upload_extracted():
    """
    Use esse comando para enviar o diretório extraído (após o processamento básico) para o Google Cloud Storage
    """
    timestamp = get_last_modified_date()
    typer.echo(f"Enviando diretório para GCS com data: {timestamp}")
    subprocess.check_output(
        f"gsutil -o GSUtil:parallel_composite_upload_threshold=300M -m cp extracted/*.csv gs://driva-lake/crawlers/RFB/{timestamp}/extracted",
        shell=True,
    )


@app.command()
def clean():
    """
    Use esse comando para apagar qualquer dado gerado pela CLI.
    """
    typer.echo("Limpando diretórios")
    shutil.rmtree("extracted")
    shutil.rmtree("zip")


@app.command()
def all():
    """
    Use esse comando para baixar, extrair e combinar todos os dados.
    """
    # download()
    # upload_zip()
    # extract()
    # combine()
    upload_extracted()
    clean()


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['driva_rfb']

package_data = \
{'': ['*'], 'driva_rfb': ['scripts/*']}

install_requires = \
['beautifulsoup4>=4.10.0,<5.0.0',
 'httpx>=0.22.0,<0.23.0',
 'pywget>=0.31,<0.32',
 'requests>=2.27.1,<3.0.0',
 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['driva-rfb = driva_rfb.main:app']}

setup_kwargs = {
    'name': 'driva-rfb',
    'version': '1.0.0',
    'description': '',
    'long_description': '# Driva-RFB \nCLI para ajudar a interagir com a base da Receita Federal Brasileira',
    'author': 'Victor Belinello',
    'author_email': 'victor@datadriva.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import subprocess
from datetime import datetime
from multiprocessing import Pool
from pathlib import Path
from typing import List, Set

import requests
from bs4 import BeautifulSoup
from pywget import wget

URL = "http://200.152.38.155/CNPJ/"
DOWNLOAD_FOLDER = "zip"


def get_last_modified_date() -> str:
    html = requests.get(URL).text
    soup = BeautifulSoup(html, "html.parser")

    last_modified = soup.find_all("tr")[3].find_all("td")[2].text.strip()
    last_modified = datetime.strptime(last_modified, "%Y-%m-%d %H:%M").date()

    last_modified = last_modified.strftime("%Y-%m-%d")
    return last_modified


def get_download_links() -> List[str]:
    html = requests.get(URL).text
    soup = BeautifulSoup(html, "html.parser")
    hrefs = [a.get("href") for a in soup.find_all("a")]
    urls = [URL + href for href in hrefs if href.endswith(".zip")]
    return urls


def get_dates_seen() -> Set[str]:
    already_seen = (
        subprocess.check_output("gsutil ls gs://driva-lake/crawlers/RFB".split())
        .decode("utf-8")
        .split("\n")
    )
    already_seen = [
        seen.replace("gs://driva-lake/crawlers/RFB/", "") for seen in already_seen
    ]
    filtered = set()
    for date in already_seen:
        if date:
            filtered.add(date.split("/")[0])
    return filtered


def has_new_crawl() -> bool:
    return not get_last_modified_date() in get_dates_seen()


def _download(url: str):
    wget.download(url, DOWNLOAD_FOLDER + "/")


def check_if_has_tmp() -> List[str]:
    links = []
    for file in Path(__file__).parents[1].glob("*.tmp"):
        filename = file.name
        filename = filename.replace(".tmp", "")
        links.append(f"http://200.152.38.155/CNPJ/{filename}")
    return links


def download_all(restart: bool):
    links = get_download_links()
    tmp_links = check_if_has_tmp()
    if tmp_links and restart:
        links = tmp_links
    downloaded_files = set(Path(DOWNLOAD_FOLDER).glob("*.zip"))
    if len(downloaded_files) == 37:
        print(
            f"37 arquivos já foram baixados. Apague a pasta {DOWNLOAD_FOLDER} caso queira baixar novamente."
        )
        return
    # shutil.rmtree(DOWNLOAD_FOLDER, ignore_errors=True)
    # os.makedirs(DOWNLOAD_FOLDER)

    with Pool() as p:
        p.map(_download, links)
